@@ -1,21 +1,25 @@
-use std::cmp::max;
+use std::cmp::{max, min};
+use std::error::Error;
 use std::fmt;
-use std::fmt::Display;
+use std::fmt::Debug;
 use std::marker::PhantomData;
 
 use crate::bit_reader::BitReader;
 use crate::bits;
-use crate::prefix::Prefix;
+use crate::constants::*;
+use crate::errors::{HeaderDtypeError, MagicHeaderError};
+use crate::prefix::{Prefix, PrefixDecompressionInfo};
 use crate::types::{DataType, NumberLike};
 use crate::utils;
-use crate::utils::{BITS_TO_ENCODE_N_ENTRIES, BITS_TO_ENCODE_PREFIX_LEN, MAGIC_HEADER, MAX_MAX_DEPTH};
 
+#[derive(Clone)]
 pub struct Decompressor<T, DT> where T: NumberLike, DT: DataType<T> {
   prefixes: Vec<Prefix<T>>,
-  prefix_map: Vec<Option<Prefix<T>>>,
+  prefix_map: Vec<PrefixDecompressionInfo<T>>,
   prefix_len_map: Vec<u32>,
   max_depth: u32,
   n: usize,
+  is_single_prefix: bool,
   data_type: PhantomData<DT>,
 }
 
@@ -29,39 +33,41 @@ impl<T, DT> Decompressor<T, DT> where T: NumberLike, DT: DataType<T> {
     let mut prefix_map = Vec::with_capacity(n_pref);
     let mut prefix_len_map = Vec::with_capacity(n_pref);
     for _ in 0..n_pref {
-      prefix_map.push(None);
+      prefix_map.push(PrefixDecompressionInfo::new());
       prefix_len_map.push(u32::MAX);
     }
     for p in &prefixes {
       let i = bits::bits_to_usize_truncated(&p.val, max_depth);
-      prefix_map[i] = Some(p.clone());
+      prefix_map[i] = p.into();
       prefix_len_map[i] = p.val.len() as u32;
     }
 
+    let is_single_prefix = prefixes.len() == 1;
     Decompressor {
       prefixes,
       prefix_map,
       prefix_len_map,
       max_depth,
       n,
+      is_single_prefix,
       data_type: PhantomData,
     }
   }
 
-  pub fn from_reader(bit_reader: &mut BitReader) -> Result<Self, String> {
-    match bit_reader.read_bytes(MAGIC_HEADER.len()) {
-      Ok(bytes) => {
-        if bytes != MAGIC_HEADER {
-          return Err(format!(
-            "file header '{:?}' does not match expected magic header '{:?}'",
-            bytes,
-            MAGIC_HEADER,
-          ));
-        }
-      },
-      Err(s) => {
-        return Err(s);
-      },
+  pub fn from_reader(bit_reader: &mut BitReader) -> Result<Self, Box<dyn Error>> {
+    let bytes = bit_reader.read_bytes(MAGIC_HEADER.len())?;
+    if bytes != MAGIC_HEADER {
+      return Err(Box::new(MagicHeaderError {
+        header: bytes.to_vec()
+      }));
+    }
+    let bytes = bit_reader.read_bytes(1)?;
+    let byte = bytes[0];
+    if byte != DT::HEADER_BYTE {
+      return Err(Box::new(HeaderDtypeError {
+        dtype_byte: byte,
+        expected_byte: DT::HEADER_BYTE,
+      }));
     }
 
     let n = bit_reader.read_u64(BITS_TO_ENCODE_N_ENTRIES as usize) as usize;
@@ -74,72 +80,75 @@ impl<T, DT> Decompressor<T, DT> where T: NumberLike, DT: DataType<T> {
       let upper = DT::from_bytes(bits::bits_to_bytes(upper_bits));
       let code_len = bit_reader.read_u64(BITS_TO_ENCODE_PREFIX_LEN as usize) as usize;
       let val = bit_reader.read(code_len);
-      prefixes.push(Prefix::new(val, lower, upper, DT::u64_diff(upper, lower)));
+      let jumpstart = if bit_reader.read_one() {
+        Some(bit_reader.read_u64(BITS_TO_ENCODE_JUMPSTART as usize) as usize)
+      } else {
+        None
+      };
+      prefixes.push(Prefix::new(val, lower, upper, DT::offset_diff(upper, lower), jumpstart));
     }
 
     let decompressor = Decompressor::new(prefixes, n);
 
-    return Ok(decompressor);
+    Ok(decompressor)
   }
 
   pub fn decompress(&self, reader: &mut BitReader) -> Vec<T> {
     self.decompress_n(reader, self.n)
   }
 
-  pub fn decompress_n(&self, reader: &mut BitReader, n: usize) -> Vec<T> {
-    let pow = 1_usize << self.max_depth;
-    let mut res = Vec::with_capacity(n);
-    // handle the case when there's just one prefix of length 0
-    let default_lower;
-    let default_upper;
-    let default_k;
-    match &self.prefix_map[0] {
-      Some(p) if p.val.len() == 0 => {
-        default_lower = p.lower;
-        default_upper = p.upper;
-        default_k = p.k;
-      },
-      _ => {
-        // any usage of these should be unreachable
-        default_lower = DT::ZERO;
-        default_upper = DT::ZERO;
-        default_k = 0;
-      }
-    };
-    for _ in 0..n {
-      let mut lower = default_lower;
-      let mut upper = default_upper;
-      let mut k = default_k;
+  fn next_prefix(&self, reader: &mut BitReader) -> PrefixDecompressionInfo<T> {
+    if self.is_single_prefix {
+      self.prefix_map[0]
+    } else {
       let mut prefix_idx = 0;
       for prefix_len in 1..self.max_depth + 1 {
         if reader.read_one() {
-          prefix_idx |= pow >> prefix_len;
+          prefix_idx |= 1 << (self.max_depth - prefix_len);
         }
         if self.prefix_len_map[prefix_idx] == prefix_len {
-          let p = self.prefix_map[prefix_idx].as_ref().unwrap();
-          lower = p.lower;
-          upper = p.upper;
-          k = p.k;
-          break;
+          return self.prefix_map[prefix_idx];
         }
       }
-      let range = DT::u64_diff(upper, lower);
-      let mut offset = reader.read_u64(k as usize);
-      if k < 64 {
-        let most_significant = 1_u64 << k;
-        if range - offset >= most_significant {
-          if reader.read_one() {
-            offset += most_significant;
+      panic!("prefixes are corrupt");
+    }
+  }
+
+  pub fn decompress_n(&self, reader: &mut BitReader, n: usize) -> Vec<T> {
+    let mut res = Vec::with_capacity(n);
+    let mut i = 0;
+    while i < n {
+      let p = self.next_prefix(reader);
+      let range = DT::offset_diff(p.upper, p.lower);
+
+      let reps = match p.run_len_jumpstart {
+        None => {
+          1
+        },
+        Some(jumpstart) => {
+          // we stored the number of occurrences minus 1
+          // because we knew it's at least 1
+          min(reader.read_varint(jumpstart) + 1, n - i)
+        },
+      };
+
+      for _ in 0..reps {
+        let mut offset = reader.read_u64(p.k as usize);
+        if p.k < 64 {
+          let most_significant = 1_u64 << p.k;
+          if range - offset >= most_significant && reader.read_one() {
+            offset |= most_significant;
           }
         }
+        res.push(DT::add_offset(p.lower, offset));
       }
-      res.push(DT::add_u64(lower, offset));
+      i += reps;
     }
     res
   }
 }
 
-impl<T, DT> Display for Decompressor<T, DT> where T: NumberLike, DT: DataType<T> {
+impl<T, DT> Debug for Decompressor<T, DT> where T: NumberLike, DT: DataType<T> {
   fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
     utils::display_prefixes(&self.prefixes, f)
   }
