@@ -11,6 +11,8 @@ pub use types::unsigned32::U32Compressor;
 pub use types::unsigned32::U32Decompressor;
 pub use types::unsigned64::U64Compressor;
 pub use types::unsigned64::U64Decompressor;
+pub use types::boolean::BoolCompressor;
+pub use types::boolean::BoolDecompressor;
 
 pub use constants::MAX_ENTRIES;
 
